/* -*-c-*-
 *
 * $Id: scan.c,v 1.1 1999/07/01 08:56:23 mdw Exp $
 *
 * Character scanners
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: scan.c,v $
 * Revision 1.1  1999/07/01 08:56:23  mdw
 * Initial revision
 *
 */

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <mLib/dstr.h>

#include "scan.h"

/*----- Main code ---------------------------------------------------------*/

/* --- Generic scanner setup --- */

static void scan_init(scanner *sc, scan_ops *ops, const char *src)
{
  sc->ops = ops;
  sc->line = 1;
  sc->src = src;
  dstr_create(&sc->d);
}

/* --- @argv@ scanner --- */

static int argv_scan(void *p)
{
  scan_argvctx *c = p;
  int ch;

  if (c->ch != EOF) {
    ch = c->ch;
    c->ch = EOF;
  } else if (*c->p)
    ch = *c->p++;
  else if (*c->pp) {
    c->p = *c->pp++;
    ch = '\n';
  } else
    ch = EOF;

  return (ch);
}

static void argv_unscan(int ch, void *p)
{
  scan_argvctx *c = p;
  c->ch = ch;
}

void scan_argvinit(scan_argvctx *c, char **pp)
{
  static struct scan_ops ops = { argv_scan, argv_unscan };
  c->p = *pp++;
  c->pp = pp;
  c->ch = EOF;
  scan_init(&c->sc, &ops, "<args>");
}

/* --- File scanner --- */

static int file_scan(void *p)
{
  scan_filectx *c = p;
  return (getc(c->fp));
}

static void file_unscan(int ch, void *p)
{
  scan_filectx *c = p;
  ungetc(ch, c->fp);
}

void scan_fileinit(scan_filectx *c, FILE *fp, const char *file)
{
  static struct scan_ops ops = { file_scan, file_unscan };
  c->fp = fp;
  scan_init(&c->sc, &ops, file);
}

/* --- Miscellaneous functions --- */

void scan_destroy(void *p)
{
  scanner *sc = p;
  dstr_destroy(&sc->d);
}

/*----- That's all, folks -------------------------------------------------*/
