/* -*-c-*-
 *
 * $Id: scan.h,v 1.1 1999/07/01 08:56:23 mdw Exp $
 *
 * Character scanners
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: scan.h,v $
 * Revision 1.1  1999/07/01 08:56:23  mdw
 * Initial revision
 *
 */

#ifndef SCAN_H
#define SCAN_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>

#include <mLib/dstr.h>

/*----- Data structures ---------------------------------------------------*/

/* --- A configuration scanner --- */

typedef struct scan_ops {
  int (*scan)(void */*p*/);		/* Character scan function */
  void (*unscan)(int /*ch*/, void */*p*/); /* Character pushback function */
} scan_ops;

typedef struct scanner {
  scan_ops *ops;			/* Pointer to scanner operations */
  const char *src;			/* Name of this source */
  int line;				/* Current line number */
  int t;				/* Token type */
  dstr d;				/* Current token value */
} scanner;

/*----- Scanners provided -------------------------------------------------*/

/* --- The @argv@ scanner --- */

typedef struct scan_argvctx {
  scanner sc;				/* Scanner base structure */
  char *p;				/* Pointer to next character */
  char **pp;				/* Pointer to next string */
  int ch;				/* Pushback character */
} scan_argvctx;

extern void scan_argvinit(scan_argvctx */*c*/, char **/*pp*/);

/* --- The file scanner --- */

typedef struct scan_filectx {
  scanner sc;				/* Scanner base structure */
  FILE *fp;				/* Stream pointer */
} scan_filectx;

extern void scan_fileinit(scan_filectx */*c*/,
			  FILE */*fp*/, const char */*file*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
